@@ -10,6 +10,9 @@ binmode STDOUT => ":bytes";
 sub new{
 	my $pkg = shift;
 	my $data = {
+	total_results => undef,
+	total_pages => undef,
+	current_page => undef,
 	collection => []
 	};
 	bless $data, $pkg;
@@ -26,6 +29,21 @@ sub add_Amazon{
 	push @{$self->{collection}}, $add_data;
 }
 
+sub total_results {
+	my $self = shift;
+	return $self->{total_results};
+}
+
+sub total_pages {
+	my $self = shift;
+	return $self->{total_pages};
+}
+
+sub current_page {
+	my $self = shift;
+	return $self->{current_page};
+}
+
 sub collection{
 	my $self = shift;
 	my @list;
